#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace ElementalGame
{
    public class Vec2
    {
        public Texture2D read;
        public Texture2D write;
        public EffectParameter gpuPointer;
        private Effect thisEffect;
        private List<Vector2> properties = new List<Vector2>();
        public RenderTarget2D writeRenderTarget;
        public RenderTarget2D readRenderTarget;

        private string outTexName;

        int count;


        public Vec2(Effect effect, string gpuParameterName, string textureName)
        {
            outTexName = textureName;

            thisEffect = effect;
            GraphicsDevice gd = ElementalGame.graphics.GraphicsDevice;

            // Create Render Targets
            writeRenderTarget = new RenderTarget2D(gd, GpuProcessor.INDEXED_TARGET_SIZE, GpuProcessor.INDEXED_TARGET_SIZE, 1, SurfaceFormat.Vector4);
            readRenderTarget = new RenderTarget2D(gd, GpuProcessor.INDEXED_TARGET_SIZE, GpuProcessor.INDEXED_TARGET_SIZE, 1, SurfaceFormat.Vector4);

            // Clear out textures
            gd.SetRenderTarget(0, readRenderTarget);
            gd.ResolveRenderTarget(0);
            read = readRenderTarget.GetTexture();

            gd.SetRenderTarget(0, writeRenderTarget);
            gd.ResolveRenderTarget(0);
            write = writeRenderTarget.GetTexture();

            gd.SetRenderTarget(0, null);

            try
            {
                // Set pointer to shader variable
                gpuPointer = thisEffect.Parameters[gpuParameterName];
            }
            catch
            {
                throw new InvalidOperationException("Parameter not found.");
            }

        }

        public void AddProperty(Vector2 property)
        {
            properties.Add(property + Vector2.Zero);
        }
        
        public void SetTexData(float minValue, float maxValue)
        {
            // Get the data range
            float range = maxValue - minValue;

            // Set GPU Object Count
            count = properties.Count;
            Vector4[] texValues = new Vector4[GpuProcessor.INDEXED_TARGET_SIZE * GpuProcessor.INDEXED_TARGET_SIZE];

            // Convert Position
            int texCounter = 0;
            for (int i = 0; i < properties.Count; i++)
            {
                texValues[texCounter].X = (properties[i].X - minValue) / range;
                texValues[texCounter].Y = (properties[i].Y - minValue) / range;
                texValues[texCounter].Z = 0.5f;
                texValues[texCounter].W = 1.0f;
                texCounter++;
            }
            read.SetData<Vector4>(texValues);
            write.SetData<Vector4>(texValues);
            gpuPointer.SetValue(read);
        }

        public void SwapBuffers()
        {
            // Swap the textures
            Texture2D temp = read;
            read = write;
            write = temp;

            // Swap the render targets
            RenderTarget2D tempTarget = readRenderTarget;
            readRenderTarget = writeRenderTarget;
            writeRenderTarget = tempTarget;

            // Reset GPU Variable
            gpuPointer.SetValue(read);
        }

        public void WriteToFile()
        {
            write.Save(outTexName + "_Read.png", ImageFileFormat.Png);
            read.Save(outTexName + "_Write.png", ImageFileFormat.Png);
        }
    }
}
