
namespace ElementalGame
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    #endregion

    public class GpuObject
    {
        Effect effect;

        // Object Variables
        public Vec3 currentPosition;
        public Quat rotation;
        public Vec1 scale;
        public Vec1 mass;
        public Vec3 velocity;
        public Vec3 acceleration;
        public Vec3 lastPosition;
        public Vec3 totalForce;
        public Vec1 radius;
        public Vec3 dummy;

        int INDEXED_MAPS_SCALE = 1;
        public int numObjects = 0;


        public GpuObject()
        {
            effect = ElementalGame.content.Load<Effect>("PhysicsGPU/GpuCollisionPhysics");

            // Spacial Data Structure
            currentPosition = new Vec3(effect, "CurrentPosition", "position");
            rotation = new Quat(effect, "Rotation", "rotation");
            scale = new Vec1(effect, "Scale", "scale");
            mass = new Vec1(effect, "Mass", "mass");
            velocity = new Vec3(effect, "Velocity", "velocity");
            acceleration = new Vec3(effect, "Acceleration", "acceleration");
            lastPosition = new Vec3(effect, "LastPosition", "lastPosition");
            totalForce = new Vec3(effect, "TotalForce", "totalForce");
            radius = new Vec1(effect, "Radius", "radius");
            dummy = new Vec3(effect, "Dummy", "dummy");
        }

        public void ConvertToTextures(List<GameObject> gpuGameObjects)
        {
            numObjects = gpuGameObjects.Count;

            // Radius
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                radius.AddProperty(gpuGameObjects[i].boundingSphere.Radius);
            }
            radius.SetTexData(0, 100);
            radius.WriteToFile();

            // Scale
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                scale.AddProperty(gpuGameObjects[i].mass.scale);
            }
            scale.SetTexData(0, 100);
            scale.WriteToFile();

            // Mass
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                mass.AddProperty(gpuGameObjects[i].mass.mass);
            }
            mass.SetTexData(0, 100);
            mass.WriteToFile();

            // CurrentPosition
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                currentPosition.AddProperty(gpuGameObjects[i].mass.currentPosition);
            }
            currentPosition.SetTexData(-Terrain.collisionMapSize, Terrain.collisionMapSize);
            currentPosition.WriteToFile();

            // Rotation (0-1?)
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                rotation.AddProperty(gpuGameObjects[i].mass.currentRotation);
            }
            rotation.SetTexData(-100, 100);
            rotation.WriteToFile();

            // Velocity = 0
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                velocity.AddProperty(gpuGameObjects[i].mass.velocity);
            }
            velocity.SetTexData(-Terrain.collisionMapSize, Terrain.collisionMapSize);
            velocity.WriteToFile();
            
            // Acceleration = 0
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                acceleration.AddProperty(gpuGameObjects[i].mass.acceleration);
            }
            acceleration.SetTexData(-Terrain.collisionMapSize, Terrain.collisionMapSize);
            acceleration.WriteToFile();
            
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                lastPosition.AddProperty(gpuGameObjects[i].mass.lastPosition);
            }
            lastPosition.SetTexData(-Terrain.collisionMapSize, Terrain.collisionMapSize);
            lastPosition.WriteToFile();
            
            // TotalForce = 0
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                totalForce.AddProperty(gpuGameObjects[i].mass.totalForce);
            }
            totalForce.SetTexData(-Terrain.collisionMapSize, Terrain.collisionMapSize);
            totalForce.WriteToFile();

            // Dummy Variable
            for (int i = 0; i < gpuGameObjects.Count; i++)
            {
                dummy.AddProperty(gpuGameObjects[i].mass.currentPosition);
            }
            dummy.SetTexData(-Terrain.collisionMapSize, Terrain.collisionMapSize);
            dummy.WriteToFile();
        }

        public void RetrieveCurrentPositions(ref List<GameObject> gameObjectList)
        {
            Vector4[] currentPositionArray = new Vector4[GpuProcessor.INDEXED_TARGET_SIZE * GpuProcessor.INDEXED_TARGET_SIZE];

            currentPosition.write.GetData<Vector4>(currentPositionArray);

            for (int i = 0; i < numObjects; i++)
            {
                gameObjectList[i].mass.currentPosition = currentPosition.UnScale(currentPositionArray[i]);
            }
        }
    }
}